@@ -9,6 +9,7 @@ import urllib.request
 import urllib.parse
 import urllib.error
 
+from enum import IntEnum
 from http.cookiejar import CookieJar
 
 
@@ -21,6 +22,12 @@ class Tado:
                    t.getClimate(1) # Get climate, zone 1.
     """
 
+    """Constants needed for get/set Schedule and Timetable"""
+    class Timetable(IntEnum):
+        ONE_DAY = 0
+        THREE_DAY = 1
+        SEVEN_DAY = 2
+
     _debugCalls = False
 
     # Instance-wide constant info
@@ -29,6 +36,7 @@ class Tado:
     mobi2url = 'https://my.tado.com/mobile/1.9/'
     refresh_token = ''
     refresh_at = datetime.datetime.now() + datetime.timedelta(minutes=5)
+    timeout = 10
 
     # 'Private' methods for use in class, Tado mobile API V1.9.
     def _mobile_apiCall(self, cmd):
@@ -42,7 +50,7 @@ class Tado:
 
         url = '%s%s' % (self.mobi2url, cmd)
         req = urllib.request.Request(url, headers=self.headers)
-        response = self.opener.open(req)
+        response = self.opener.open(req, timeout=self.timeout)
         str_response = response.read().decode('utf-8')
 
         if self._debugCalls:
@@ -78,7 +86,7 @@ class Tado:
                                      method=method,
                                      data=data)
 
-        response = self.opener.open(req)
+        response = self.opener.open(req, timeout=self.timeout)
 
         if self._debugCalls:
             _LOGGER.debug("api call: %s: %s, response %s",
@@ -125,7 +133,7 @@ class Tado:
                                      headers={'Content-Type': 'application/json',
                                               'Referer' : 'https://my.tado.com/'})
 
-        response = self.opener.open(req)
+        response = self.opener.open(req, timeout=self.timeout)
         str_response = response.read().decode('utf-8')
 
         self._setOAuthHeader(json.loads(str_response))
@@ -151,7 +159,7 @@ class Tado:
                                      headers={'Content-Type': 'application/json',
                                               'Referer' : 'https://my.tado.com/'})
 
-        response = self.opener.open(req)
+        response = self.opener.open(req, timeout=self.timeout)
         str_response = response.read().decode('utf-8')
 
         self._setOAuthHeader(json.loads(str_response))
@@ -168,7 +176,7 @@ class Tado:
 
         url = 'https://my.tado.com/api/v2/me'
         req = urllib.request.Request(url, headers=self.headers)
-        response = self.opener.open(req)
+        response = self.opener.open(req, timeout=self.timeout)
         str_response = response.read().decode('utf-8')
         data = json.loads(str_response)
         return data
@@ -197,6 +205,17 @@ class Tado:
         data = self._apiCall(cmd)
         return data
 
+    def getHomeState(self):
+        """Gets current state of Home."""
+        # returns {"presence":"AWAY"} or {"presence":"HOME"}
+        # without an auto assist skill presence is not switched automatically,
+        # but a button is shown in the app. showHomePresenceSwitchButton
+        # is an indicator, that the homeState can be switched
+        # {"presence":"HOME","showHomePresenceSwitchButton":true}
+        cmd = 'state'
+        data = self._apiCall(cmd)
+        return data
+    
     def getCapabilities(self, zone):
         """Gets current capabilities of Zone zone."""
         # pylint: disable=C0103
@@ -213,6 +232,78 @@ class Tado:
         return {'temperature' : data['insideTemperature']['celsius'],
                 'humidity'    : data['humidity']['percentage']}
 
+    def getTimetable(self, zone):
+        """Get the Timetable type currently active"""
+        # pylint: disable=C0103
+
+        cmd = 'zones/%i/schedule/activeTimetable' % (zone)
+
+        data = self._apiCall(cmd, "GET", {}, True)
+
+        if "id" in data:
+            return Tado.Timetable(data["id"])
+
+        raise Exception('Returned data did not contain "id" : '+str(data))
+
+    def getHistoric(self, zone, date):
+        """Gets historic information on given date for Zone zone."""
+        try:
+            day = datetime.datetime.strptime(date, '%Y-%m-%d')
+        except ValueError:
+            raise ValueError("Incorrect date format, should be YYYY-MM-DD")
+
+        cmd = 'zones/%i/dayReport?date=%s' % (zone, day.strftime('%Y-%m-%d'))
+        data = self._apiCall(cmd)
+        return data
+
+    def setTimetable(self, zone, id):
+        """Set the Timetable type currently active
+           id = 0 : ONE_DAY (MONDAY_TO_SUNDAY)
+           id = 1 : THREE_DAY (MONDAY_TO_FRIDAY, SATURDAY, SUNDAY)
+           id = 3 : SEVEN_DAY (MONDAY, TUESDAY, WEDNESDAY ...)"""
+        # pylint: disable=C0103
+
+        # Type checking
+        if not isinstance(id, Tado.Timetable):
+            raise TypeError('id must be an instance of Tado.Timetable')
+
+        cmd = 'zones/%i/schedule/activeTimetable' % (zone)
+
+        data = self._apiCall(cmd, "PUT", {'id': id }, True)
+        return data
+
+    def getSchedule(self, zone, id, day=None):
+        """Get the JSON representation of the schedule for a zone
+           a Zone has 3 different schedules, one for each timetable
+           (see setTimetable) """
+        # pylint: disable=C0103
+
+        # Type checking
+        if not isinstance(id, Tado.Timetable):
+            raise TypeError('id must be an instance of Tado.Timetable')
+
+        if day:
+            cmd = 'zones/%i/schedule/timetables/%i/blocks/%s' % (zone,id,day)
+        else:
+            cmd = 'zones/%i/schedule/timetables/%i/blocks' % (zone,id)
+
+        data = self._apiCall(cmd, "GET", {}, True)
+        return data
+
+
+    def setSchedule(self, zone, id, day, data):
+        """Set the schedule for a zone, day is required"""
+        # pylint: disable=C0103
+
+        # Type checking
+        if not isinstance(id, Tado.Timetable):
+            raise TypeError('id must be an instance of Tado.Timetable')
+
+        cmd = 'zones/%i/schedule/timetables/%i/blocks/%s' % (zone,id,day)
+
+        data = self._apiCall(cmd, "PUT", data, True)
+        return data
+
     def getWeather(self):
         """Gets outside weather data"""
         # pylint: disable=C0103
@@ -236,7 +327,14 @@ class Tado:
         cmd = 'getAppUsersRelativePositions'
         data = self._mobile_apiCall(cmd)
         return data
+    
+    def getMobileDevices(self):
+        """Gets information about mobile devices"""
 
+        cmd = 'mobileDevices'
+        data = self._apiCall(cmd)
+        return data
+    
     def resetZoneOverlay(self, zone):
         """Delete current overlay"""
         # pylint: disable=C0103
@@ -287,6 +385,49 @@ class Tado:
         data = self._apiCall(cmd, "PUT", post_data)
         return data
 
+    def setHome(self):
+        """Sets HomeState to HOME """
+        # it seems, this can be set anytime
+        cmd = 'presence'
+        payload = '{"homePresence":"HOME"}'
+        data = self._apiCall(cmd, "PUT", payload)
+        return data
+
+    def setAway(self):
+        """Sets HomeState to AWAY """
+        # this can only be set if everybody left the home and 
+        # showHomePresenceSwitchButton = true
+        cmd = 'presence'
+        payload = '{"homePresence":"AWAY"}'
+        data = self._apiCall(cmd, "PUT", payload)
+        return data
+    
+    def getWindowState(self, zone):
+        """Returns the state of the window for Zone zone"""
+        data = self.getState(zone)['openWindow']
+        return {'openWindow' : data}
+
+    def getOpenWindowDetected(self, zone):
+        """Returns whether an open window is detected."""
+        data = self.getState(zone)
+        if "openWindowDetected" in data:
+            return {'openWindowDetected': data['openWindowDetected']}
+        else:
+            return {'openWindowDetected': False}
+
+    def setOpenWindow(self, zone):
+        """Sets the window in Zone zone to open"""
+        # this can only be set if an open window was detected in this zone
+        cmd = 'zones/%i/state/openWindow/activate' % (zone)
+        data = self._apiCall(cmd, "POST", {}, True)
+        return data
+
+    def resetOpenWindow(self, zone):
+        """Sets the window in zone Zone to closed"""
+        cmd = 'zones/%i/state/openWindow' % zone
+        data = self._apiCall(cmd, "DELETE", {}, True)
+        return data
+    
     # Ctor
     def __init__(self, username, password):
         """Performs login and save session cookie."""
